package com.ruoyi.dashboard.service;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.ruoyi.common.domain.vo.LocalDateDecimalVO;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.MathUtils;
import com.ruoyi.common.utils.collection.CollectionUtils;
import com.ruoyi.dashboard.domain.dto.BusinessStatisticsQuery;
import com.ruoyi.dashboard.domain.dto.ServiceIncomeQuery;
import com.ruoyi.dashboard.domain.vo.BonusTrendDetailVO;
import com.ruoyi.dashboard.domain.vo.BonusTrendVO;
import com.ruoyi.dashboard.domain.vo.BusinessStatisticsByStoreVO;
import com.ruoyi.dashboard.domain.vo.BusinessStatisticsVO;
import com.ruoyi.dashboard.domain.vo.DashboardBonusGroupByMchVO;
import com.ruoyi.dashboard.domain.vo.MchRevenueVO;
import com.ruoyi.dashboard.domain.vo.ServiceIncomeVO;
import com.ruoyi.dashboard.domain.vo.TodoListVO;
import com.ruoyi.dashboard.mapper.DashboardMapper;
import com.ruoyi.ss.abnormal.domain.AbnormalQuery;
import com.ruoyi.ss.abnormal.domain.enums.AbnormalStatus;
import com.ruoyi.ss.abnormal.service.AbnormalService;
import com.ruoyi.ss.bonus.domain.BonusQuery;
import com.ruoyi.ss.bonus.domain.BonusVO;
import com.ruoyi.ss.bonus.domain.enums.BonusArrivalType;
import com.ruoyi.ss.bonus.domain.enums.BonusStatus;
import com.ruoyi.ss.bonus.domain.vo.BonusDailyAmountVO;
import com.ruoyi.ss.bonus.service.BonusService;
import com.ruoyi.ss.bonusRefund.domain.BonusRefundQuery;
import com.ruoyi.ss.bonusRefund.service.BonusRefundService;
import com.ruoyi.ss.complaint.domain.SmComplaintQuery;
import com.ruoyi.ss.complaint.service.ISmComplaintService;
import com.ruoyi.ss.device.domain.DeviceQuery;
import com.ruoyi.ss.device.domain.enums.DeviceStatus;
import com.ruoyi.ss.device.service.DeviceService;
import com.ruoyi.ss.mchApply.domain.MchApplyQuery;
import com.ruoyi.ss.mchApply.domain.enums.MchApplyStatus;
import com.ruoyi.ss.mchApply.service.IMchApplyService;
import com.ruoyi.ss.payBill.domain.PayBillQuery;
import com.ruoyi.ss.payBill.domain.enums.PayBillBstType;
import com.ruoyi.ss.payBill.domain.enums.PayBillStatus;
import com.ruoyi.ss.payBill.service.PayBillService;
import com.ruoyi.ss.receiveBill.domain.ReceiveBillQuery;
import com.ruoyi.ss.receiveBill.domain.enums.ReceiveBillGroupBy;
import com.ruoyi.ss.receiveBill.domain.enums.ReceiveBillType;
import com.ruoyi.ss.receiveBill.domain.vo.ReceiveAmountVO;
import com.ruoyi.ss.receiveBill.service.ReceiveBillService;
import com.ruoyi.ss.reconciliationDate.domain.ReconciliationDate;
import com.ruoyi.ss.recordBalance.service.RecordBalanceService;
import com.ruoyi.ss.refund.domain.RefundQuery;
import com.ruoyi.ss.refund.domain.enums.RefundStatus;
import com.ruoyi.ss.refund.service.RefundService;
import com.ruoyi.ss.riskInfo.domain.RiskInfoQuery;
import com.ruoyi.ss.riskInfo.domain.enums.RiskInfoStatus;
import com.ruoyi.ss.riskInfo.service.RiskInfoService;
import com.ruoyi.ss.storeApply.domain.StoreApplyQuery;
import com.ruoyi.ss.storeApply.domain.enums.StoreApplyStatus;
import com.ruoyi.ss.storeApply.service.StoreApplyService;
import com.ruoyi.ss.transactionBill.domain.TransactionBillQuery;
import com.ruoyi.ss.transactionBill.domain.enums.TransactionBillGroupBy;
import com.ruoyi.ss.transactionBill.domain.enums.TransactionBillStatus;
import com.ruoyi.ss.transactionBill.domain.enums.TransactionBillType;
import com.ruoyi.ss.transactionBill.domain.vo.TransactionAmountVO;
import com.ruoyi.ss.transactionBill.service.TransactionBillService;
import com.ruoyi.ss.user.domain.SmUserVO;
import com.ruoyi.ss.user.service.UserService;

/**
 * @author wjh
 * 2024/8/6
 */
@Service
public class DashboardService {

    @Autowired
    private DashboardMapper dashboardMapper;

    @Autowired
    private TransactionBillService transactionBillService;

    @Autowired
    private IMchApplyService mchApplyService;

    @Autowired
    private AbnormalService abnormalService;

    @Autowired
    private ISmComplaintService complaintService;

    @Autowired
    private DeviceService deviceService;

    @Autowired
    private StoreApplyService storeApplyService;

    @Autowired
    private ReceiveBillService receiveBillService;

    @Autowired
    private BonusService bonusService;

    @Autowired
    private UserService userService;

    @Autowired
    private RiskInfoService riskInfoService;

    @Autowired
    private PayBillService payBillService;

    @Autowired
    private RefundService refundService;

    @Autowired
    private RecordBalanceService recordBalanceService;

    @Autowired
    private BonusRefundService bonusRefundService;

    public TodoListVO getTodoList() {
        TodoListVO vo = new TodoListVO();

        // 提现申请数量
        TransactionBillQuery withdrawQuery = new TransactionBillQuery();
        withdrawQuery.setType(TransactionBillType.WITHDRAW.getType());
        withdrawQuery.setStatus(TransactionBillStatus.WITHDRAW_APPROVING.getStatus());
        vo.setWithdrawCount(transactionBillService.selectSimpleCount(withdrawQuery));

        // 商户合作申请
        MchApplyQuery mchApplyQuery = new MchApplyQuery();
        mchApplyQuery.setStatus(MchApplyStatus.APPROVING.getStatus());
        vo.setMchApplyCount(mchApplyService.selectCount(mchApplyQuery));

        // 店铺申请
        StoreApplyQuery storeApplyQuery = new StoreApplyQuery();
        storeApplyQuery.setStatus(StoreApplyStatus.WAIT_AUDIT.getStatus());
        vo.setStoreApplyCount(storeApplyService.selectCount(storeApplyQuery));

        // 设备故障
        AbnormalQuery abnormalQuery = new AbnormalQuery();
        abnormalQuery.setStatus(AbnormalStatus.UNREAD.getStatus());
        vo.setAbnormalCount(abnormalService.selectCount(abnormalQuery));

        // 投诉意见
        vo.setComplaintCount(complaintService.selectCount(new SmComplaintQuery()));

        // 过期设备
        DeviceQuery deviceQuery = new DeviceQuery();
        deviceQuery.setIsArrears(true);
        vo.setArrearsDeviceCount(deviceService.selectCount(deviceQuery));

        // 风控审核
        RiskInfoQuery riskInfoQuery = new RiskInfoQuery();
        riskInfoQuery.setStatus(RiskInfoStatus.WAIT_VERIFY.getStatus());
        vo.setRiskVerifyCount(riskInfoService.selectCount(riskInfoQuery));

        return vo;
    }

    public List<ServiceIncomeVO> getServiceIncome(ServiceIncomeQuery query) {

        // 查询服务费
        BonusQuery bonusQuery = new BonusQuery();
        bonusQuery.setPayDateStart(query.getStartDate());
        bonusQuery.setPayDateEnd(query.getEndDate());
        bonusQuery.setArrivalType(BonusArrivalType.PLATFORM.getType());
        List<BonusDailyAmountVO> bonusList = bonusService.selectDailyAmount(bonusQuery);

        // 查询支付单
        PayBillQuery payQuery = new PayBillQuery();
        payQuery.setPayDateStart(query.getStartDate());
        payQuery.setPayDateEnd(query.getEndDate());
        payQuery.setStatusList(PayBillStatus.payedList());
        List<LocalDateDecimalVO> payList = payBillService.selectDailyChannelCost(payQuery);

        // 查询提现
        TransactionBillQuery billQuery = new TransactionBillQuery();
        billQuery.setType(TransactionBillType.WITHDRAW.getType());
        billQuery.setStartDate(query.getStartDate());
        billQuery.setEndDate(query.getEndDate());
        billQuery.setStatusList(TransactionBillStatus.validWithdrawList());
        List<TransactionAmountVO<Date>> withdrawList = transactionBillService.selectCommonSumOfMoney(billQuery, TransactionBillGroupBy.create_date.name());

        // 查询月费收入
        ReceiveBillQuery receiveQuery = new ReceiveBillQuery();
        receiveQuery.setStartDate(query.getStartDate());
        receiveQuery.setEndDate(query.getEndDate());
        receiveQuery.setType(ReceiveBillType.MONTH.getType());
        List<ReceiveAmountVO<Date>> receiveList = receiveBillService.selectCommonSumOfAmount(receiveQuery, ReceiveBillGroupBy.create_date.name());

        // 查询订单手机号收入
        receiveQuery.setType(ReceiveBillType.MOBILE.getType());
        List<ReceiveAmountVO<Date>> mobileList = receiveBillService.selectCommonSumOfAmount(receiveQuery, ReceiveBillGroupBy.create_date.name());

        // 拼接结果集
        List<ServiceIncomeVO> result = new ArrayList<>();
        if (query.getStartDate() != null && query.getEndDate() != null) {
            result = CollectionUtils.fillVoids(result, ServiceIncomeVO::getDate, (date) -> {
                ServiceIncomeVO vo = new ServiceIncomeVO();
                vo.setDate(date);

                // 订单服务费
                BonusDailyAmountVO bonus = bonusList.stream()
                        .filter(item -> item.getKey().isEqual(date))
                        .findFirst().orElse(null);
                if (bonus != null) {
                    vo.setServiceAmount(bonus.getSum());
                } else {
                    vo.setServiceAmount(BigDecimal.ZERO);
                }

                // 渠道成本
                LocalDateDecimalVO pay = payList.stream()
                        .filter(item -> item.getKey().isEqual(date))
                        .findFirst().orElse(null);
                if (pay != null) {
                    vo.setChannelCost(pay.getValue());
                } else {
                    vo.setChannelCost(BigDecimal.ZERO);
                }

                // 提现服务费收入及成本
                TransactionAmountVO<Date> withdraw = withdrawList.stream()
                        .filter(item -> item.getKey().compareTo(DateUtils.toDate(date)) == 0)
                        .findFirst().orElse(null);
                if (withdraw != null) {
                    vo.setWithdrawServiceAmount(withdraw.getServiceAmount());
                    vo.setChannelCost(vo.getChannelCost().add(withdraw.getChannelCost()));
                } else {
                    vo.setWithdrawServiceAmount(BigDecimal.ZERO);
                }

                // 月费收入
                ReceiveAmountVO<Date> receive = receiveList.stream()
                        .filter(item -> item.getKey().compareTo(DateUtils.toDate(date)) == 0)
                        .findFirst()
                        .orElse(null);
                if (receive != null) {
                    vo.setMonthAmount(receive.getReceivedAmount());
                } else {
                    vo.setMonthAmount(BigDecimal.ZERO);
                }

                // 订单手机号收入
                ReceiveAmountVO<Date> mobile = mobileList.stream()
                        .filter(item -> item.getKey().compareTo(DateUtils.toDate(date)) == 0)
                        .findFirst()
                        .orElse(null);
                if (mobile != null) {
                    vo.setBillMobileAmount(mobile.getReceivedAmount());
                } else {
                    vo.setBillMobileAmount(BigDecimal.ZERO);
                }

                return vo;
            }, query.getStartDate(), query.getEndDate());
        }
        return result;
    }

    public BusinessStatisticsByStoreVO businessStatisticsByStore(BusinessStatisticsQuery query) {
        BusinessStatisticsByStoreVO vo = new BusinessStatisticsByStoreVO();

        // 查询列表
        query.setBillType(TransactionBillType.RECHARGE.getType());
        query.setBillStatusList(TransactionBillStatus.payedOrder());
        List<BusinessStatisticsVO> list = dashboardMapper.selectBusinessStatisticsByStore(query);
        vo.setList(list);

        // 统计数量、金额
        vo.setTotalOrder(list.stream().mapToInt(BusinessStatisticsVO::getOrderCount).sum());
        vo.setTotalOrderAmount(CollectionUtils.sumDecimal(list, BusinessStatisticsVO::getArrivalAmount));

        return vo;
    }


    public List<BusinessStatisticsVO> businessStatisticsByDevice(BusinessStatisticsQuery query) {
        query.setBillType(TransactionBillType.RECHARGE.getType());
        query.setBillStatusList(TransactionBillStatus.payedOrder());
        return dashboardMapper.selectBusinessStatisticsByDevice(query);
    }

    /**
     * 查询商户中心营收统计
     */
    public MchRevenueVO mchRevenue(Long mchId) {
        MchRevenueVO vo = new MchRevenueVO();
        LocalDate today = LocalDate.now();

        // 余额
        SmUserVO mch = userService.selectSmUserByUserId(mchId);
        vo.setBalance(mch.getBalance());

        // 营业额统计
        BonusQuery bonusQuery = new BonusQuery();
        bonusQuery.setArrivalId(mchId);
        bonusQuery.setArrivalTypes(BonusArrivalType.userList());
        bonusQuery.setStatus(BonusStatus.DIVIDEND.getStatus());
        // 总营业额
        BigDecimal totalIncome = bonusService.selectSumOfPayedAmount(bonusQuery);
        if (totalIncome == null) {
            totalIncome = BigDecimal.ZERO;
        }
        vo.setTotalIncome(totalIncome);
        // 今日营业额
        bonusQuery.setPayDate(today);
        BigDecimal todayIncome = bonusService.selectSumOfPayedAmount(bonusQuery);
        if (todayIncome == null) {
            todayIncome = BigDecimal.ZERO;
        }
        vo.setTodayIncome(todayIncome);

        // 订单数统计
        TransactionBillQuery billQuery = new TransactionBillQuery();
        billQuery.setType(TransactionBillType.RECHARGE.getType());
        billQuery.setStatusList(TransactionBillStatus.payedOrder());
        billQuery.setMchId(mchId);
        // 总订单数
        int totalOrder = transactionBillService.selectSimpleCount(billQuery);
        vo.setTotalOrder(totalOrder);
        // 今日订单数
        billQuery.setCreateDate(today);
        int todayOrder = transactionBillService.selectSimpleCount(billQuery);
        vo.setTodayOrder(todayOrder);

        // 设备统计
        DeviceQuery deviceQuery = new DeviceQuery();
        deviceQuery.setUserId(mchId);
        // 设备总数
        int deviceCount = deviceService.selectCount(deviceQuery);
        vo.setDeviceCount(deviceCount);
        // 空闲设备数量
        deviceQuery.setStatus(DeviceStatus.NORMAL.getStatus());
        int freeDeviceCount = deviceService.selectCount(deviceQuery);
        vo.setFreeDeviceCount(freeDeviceCount);
        // 使用中设备数量
        deviceQuery.setStatus(DeviceStatus.USING.getStatus());
        int usingDeviceCount = deviceService.selectCount(deviceQuery);
        vo.setUsingDeviceCount(usingDeviceCount);

        return vo;
    }

    public BonusTrendVO selectBonusTrend(BonusQuery query) {
        BonusTrendVO vo = new BonusTrendVO();

        // 查询分成列表
        List<BonusVO> bonusList = bonusService.selectBonusList(query);
        // 若是日期不为空，则构建明细列表
        List<BonusTrendDetailVO> detailList = new ArrayList<>();
        if (query.getPayDateStart() != null && query.getPayDateEnd() != null) {
            detailList = CollectionUtils.fillVoids(detailList, BonusTrendDetailVO::getDate, (date) -> {
                // 当天的分成列表
                List<BonusVO> dateBonusList = bonusList.stream()
                        .filter(item -> item.getPayTime() != null && item.getPayTime().toLocalDate().isEqual(date))
                        .collect(Collectors.toList());

                // 构造明细数据
                BonusTrendDetailVO detail = new BonusTrendDetailVO();
                detail.setDate(date);
                detail.setIncomeAmount(CollectionUtils.sumDecimal(dateBonusList, BonusVO::getPayedAmount));
                detail.setOrderCount(new HashSet<>(CollectionUtils.map(dateBonusList, BonusVO::getBillId)).size());
                return detail;
            }, query.getPayDateStart(), query.getPayDateEnd());
        }
        vo.setDetailList(detailList);

        // 收入
        BigDecimal totalIncome = CollectionUtils.sumDecimal(bonusList, BonusVO::getPayedAmount);
        if (totalIncome == null) {
            totalIncome = BigDecimal.ZERO;
        }
        vo.setTotalIncome(totalIncome);

        // 订单数
        int totalOrder = new HashSet<>(CollectionUtils.map(bonusList, BonusVO::getBillId)).size();
        vo.setTotalOrder(totalOrder);

        return vo;
    }

    public List<DashboardBonusGroupByMchVO> selectBonusGroupByMch(BonusQuery query) {
        query.setArrivalTypes(BonusArrivalType.userList());
        return dashboardMapper.selectBonusGroupByMch(query);
    }

    // 获取昨日对账数据
    public ReconciliationDate selectReconciliationDateByDate(LocalDate date) {
        if (date == null) {
            return null;
        }

        ReconciliationDate result = new ReconciliationDate();
        result.setDate(date);

        // 渠道成本
        PayBillQuery payQuery = new PayBillQuery();
        payQuery.setPayDate(date);
        payQuery.setStatusList(PayBillStatus.payedList());
        result.setChannelCost(payBillService.selectSumOfChannelCost(payQuery));

        // 充值订单金额
        payQuery.setBstType(PayBillBstType.RECHARGE_ORDER.getType());
        result.setOrderAmount(payBillService.selectSumOfAmount(payQuery));

        // VIP订单金额
        payQuery.setBstType(PayBillBstType.VIP_ORDER.getType());
        result.setVipOrderAmount(payBillService.selectSumOfAmount(payQuery));

        // 充值订单退款金额
        RefundQuery refundQuery = new RefundQuery();
        refundQuery.setCreateDate(date);
        refundQuery.setStatusList(RefundStatus.successList());
        refundQuery.setPayBillBstType(PayBillBstType.RECHARGE_ORDER.getType());
        result.setRefundAmount(refundService.selectSumOfAmount(refundQuery));

        // 用户总分成
        BonusQuery bonusQuery = new BonusQuery();
        bonusQuery.setCreateDate(date);
        bonusQuery.setStatusList(BonusStatus.validList());
        bonusQuery.setArrivalTypes(BonusArrivalType.userList());
        result.setUserBonus(bonusService.selectSumOfAmount(bonusQuery));

        // 平台总分成
        bonusQuery.setArrivalTypes(BonusArrivalType.deptList());
        result.setPlatformBonus(bonusService.selectSumOfAmount(bonusQuery));

        // 用户分成退款
        BonusRefundQuery bonusRefundQuery = new BonusRefundQuery();
        bonusRefundQuery.setCreateDate(date);
        bonusRefundQuery.setBonusArrivalTypes(BonusArrivalType.userList());
        result.setUserBonusRefund(bonusRefundService.selectSumOfAmount(bonusRefundQuery));

        // 平台分成退款
        bonusRefundQuery.setBonusArrivalTypes(BonusArrivalType.deptList());
        result.setPlatformBonusRefund(bonusRefundService.selectSumOfAmount(bonusRefundQuery));

        // 提现金额
        TransactionBillQuery withdrawQuery = new TransactionBillQuery();
        withdrawQuery.setCreateDate(date);
        withdrawQuery.setType(TransactionBillType.WITHDRAW.getType());
        withdrawQuery.setStatusList(TransactionBillStatus.validWithdrawList());
        result.setWithdrawAmount(transactionBillService.selectSumOfMoney(withdrawQuery));

        // 提现服务费
        result.setWithdrawServiceFee(transactionBillService.selectSumOfServiceCharge(withdrawQuery));

        // 应收账金额
        ReceiveBillQuery receiveQuery = new ReceiveBillQuery();
        receiveQuery.setBillDate(date);
        result.setReceiveAmount(receiveBillService.selectSumOfReceivedAmount(receiveQuery));
        
        // 总分成
        result.setTotalBonus(MathUtils.addDecimal(result.getUserBonus(), result.getPlatformBonus()));
        // 分成总退款
        result.setTotalBonusRefund(MathUtils.addDecimal(result.getUserBonusRefund(), result.getPlatformBonusRefund()));
        // 实际分成
        result.setActualBonus(MathUtils.subtractDecimal(result.getTotalBonus(), result.getTotalBonusRefund()));
        // 订单总额
        result.setOrderTotalAmount(MathUtils.addDecimal(result.getOrderAmount(), result.getVipOrderAmount()));
        // 订单实收金额
        result.setOrderReceiveAmount(MathUtils.subtractDecimal(result.getOrderTotalAmount(), result.getRefundAmount()));
        // 订单实收和实际分成差额
        result.setDifference(MathUtils.subtractDecimal(result.getOrderReceiveAmount(), result.getActualBonus()));
        // 平台收益 = 平台分成 + 应收账 + 提现服务费 - 平台分成退款 - 渠道成本
        BigDecimal add = MathUtils.addDecimal(result.getPlatformBonus(), result.getReceiveAmount(), result.getWithdrawServiceFee());
        result.setPlatformIncome(MathUtils.subtractDecimal(add, result.getPlatformBonusRefund(), result.getChannelCost()));


        return result;
    }
}

